import chimera
import numpy as np

from StructBio.HBonds import HBondDicts 
from StructBio.Utilities.hiLite import HiLiter
from chimera import runCommand

# Try 3C1A
pdbIDchars = raw_input("Type in 4 character PDB ID or PDB file name: \n")
prot = chimera.openModels.open(pdbIDchars, type="PDB")[0]

hBonds = HBondDicts(prot)

hiLiter = HiLiter(prot)
hiLiter.setVerbose(True)
runCommand("ribbon")

sumBetaDistConsecRes = 0.0     # Cumulative sum of distances between CB's in consecutive residues.
sumBetaDistPLstrands = 0.0     # Cumulative sum of distances between CB's in parallel beta strands.
sumBetaDistAPstrands = 0.0     # Cumulative sum of distances between CB's in anti-parallel beta strands.

numBetaDistConsecRes = 0       # Number of distances between CB's in consecutive residues.
numBetaDistPLstrands = 0       # Number of distances between CB's in parallel beta strands.
numBetaDistAPstrands = 0       # Number of distances between CB's in anti-parallel beta strands.

print "Depress Enter key to step through each of the calculations:"

for d_a_Pair in hBonds.hBondCategory_D.keys():
    bondCat = hBonds.hBondCategory_D[d_a_Pair]
    if bondCat == "HE" or bondCat == "NA": continue
    donorAtom = d_a_Pair[0]
    acptrAtom = d_a_Pair[1]
    # Try to get coords of beta carbon in donor residue:
    try:
        betaC_inDonorRes = donorAtom.residue.findAtom('CB')
        coordBetaC_inDonorRes = np.array(betaC_inDonorRes.coord())
    except:
        continue
    # Try to get coords of beta carbon in residue before donor residue:
    try:
        betaC_B4DonorRes = prot.residueBefore(donorAtom.residue).findAtom('CB')
        coordBetaC_B4DonorRes = np.array(betaC_B4DonorRes.coord())
    except:
        continue
    # Try to get coords of beta carbon in parallel strand:
    if bondCat == "PL":
        try:
            betaC_otherStrand = prot.residueAfter(acptrAtom.residue).findAtom('CB')
            coordBetaC_otherStrand = np.array(betaC_otherStrand.coord())
        except:
            continue
    # Try to get coords of beta carbon in anti-parallel strand:
    if bondCat == "AP":
        try:
            betaC_otherStrand = acptrAtom.residue.findAtom('CB')
            coordBetaC_otherStrand = np.array(betaC_otherStrand.coord())
        except:
            continue

    # If we get this far then we have all three atoms needed for the computation.
    hiLiter.emphasize([betaC_inDonorRes], [betaC_B4DonorRes, betaC_otherStrand],
                      "Beta carbon: ", "Consecutive and adjacent strand beta carbons: ")
    
    distConsec = np.linalg.norm(coordBetaC_inDonorRes - coordBetaC_B4DonorRes)
    sumBetaDistConsecRes += distConsec
    numBetaDistConsecRes += 1
    print "Distance between consecutive beta carbons: %6.3f" % distConsec

    distAcross = np.linalg.norm(coordBetaC_inDonorRes - coordBetaC_otherStrand)

    if bondCat == "PL":
        sumBetaDistPLstrands += distAcross
        numBetaDistPLstrands += 1
        print "Distance between beta carbons across parallel strands: %6.3f" % distAcross

    if bondCat == "AP":
        sumBetaDistAPstrands += distAcross
        numBetaDistAPstrands += 1
        print "Distance between beta carbons across anti-parallel strands: %6.3f" % distAcross

    # Make the next line into a comment to get rid of stepping.
    raw_input(" ")

    
hiLiter.clearEmphasis()

print "\n"
if numBetaDistConsecRes > 0:
    print "Average distance between consecutive beta carbons: %6.3f" % (sumBetaDistConsecRes/numBetaDistConsecRes)
    print "Number of these distances: ", numBetaDistConsecRes
else:
    print "No beta sheets detected."
if numBetaDistPLstrands > 0:
    print "Average distance between beta carbons in parallel strands: %6.3f" % (sumBetaDistPLstrands/numBetaDistPLstrands)
    print "Number of these distances: ", numBetaDistPLstrands
else:
    print "No parallel strands detected."
if numBetaDistAPstrands > 0:
    print "Average distance between beta carbons in anti-parallel strands: %6.3f" % (sumBetaDistAPstrands/numBetaDistAPstrands)
    print "Number of these distances: ", numBetaDistAPstrands
else:
    print "No anti-parallel strands detected."
























